#!/bin/bash

ERROR_TIMEOUT=60                # Секунды в одном интервале
RETRY_LIMIT=3                   # Сколько интервалов подряд с ошибками нужно для перезапуска
COOLDOWN_AFTER_RESTART=120      # Секунды на "отдых" после перезапуска

ERROR_COUNT=0
CURRENT_INTERVAL=0
IN_COOLDOWN=0

SERVICE1="wb-mqtt-serial"
SERVICE2="wb-rules"

LOG_TAG="wb-watchdog"
LOG_LEVEL="INFO"  # Можно переключать на DEBUG

log() {
    local level="$1"
    local message="$2"
    [[ "$level" == "DEBUG" && "$LOG_LEVEL" != "DEBUG" ]] && return
    logger -t "$LOG_TAG" "[$level] $message"
}

cleanup() {
    log "INFO" "Shutting down."
    kill "$JOURNAL_PID" 2>/dev/null
    exit 0
}
trap cleanup INT TERM

# Запускаем journalctl как фоновый процесс и читаем через дескриптор
exec 3< <(journalctl -u "$SERVICE2" -f --no-pager --since "now")
JOURNAL_PID=$!

watch_errors() {
    local deadline=$(( $(date +%s) + ERROR_TIMEOUT ))
    local had_error=1
    local error_num=0

    log "INFO" "Interval #$CURRENT_INTERVAL started. Waiting up to $ERROR_TIMEOUT seconds for error..."

    while (( $(date +%s) < deadline )); do
        if read -r -t 1 line <&3; then
            if [[ "$line" == *"failed to SetValue for unexisting control"* || "$line" == *"SetValue() error: can't convert control value"* ]]; then
                ((error_num++))
                log "WARN" "Error #$error_num detected in interval #$CURRENT_INTERVAL."
                had_error=0
            fi
        fi
    done

    if (( had_error == 1 )); then
        log "DEBUG" "No errors detected in interval #$CURRENT_INTERVAL."
    else
        log "INFO" "Total $error_num error(s) detected in interval #$CURRENT_INTERVAL."
    fi

    return $had_error
}

while true; do
    if (( IN_COOLDOWN == 1 )); then
        log "INFO" "Cooling down for $COOLDOWN_AFTER_RESTART seconds..."
        sleep "$COOLDOWN_AFTER_RESTART"
        ERROR_COUNT=0
        CURRENT_INTERVAL=0
        IN_COOLDOWN=0
        continue
    fi

    ((CURRENT_INTERVAL++))
    if watch_errors; then
        (( ERROR_COUNT++ ))
        log "INFO" "Error interval #$ERROR_COUNT of $RETRY_LIMIT confirmed (interval #$CURRENT_INTERVAL)."

        if (( ERROR_COUNT >= RETRY_LIMIT )); then
            log "ERROR" "Reached $RETRY_LIMIT error intervals. Restarting services..."

            timeout 30 systemctl restart "$SERVICE1" || log "ERROR" "Failed to restart $SERVICE1"
            timeout 30 systemctl restart "$SERVICE2" || log "ERROR" "Failed to restart $SERVICE2"

            IN_COOLDOWN=1
        fi
    else
        if (( ERROR_COUNT > 0 )); then
            log "INFO" "Error did not persist. Resetting error counter."
        fi
        ERROR_COUNT=0
    fi
done
